package polyglot.ext.polyj.ast;

import polyglot.ext.polyj.util.*;
import polyglot.ext.polyj.types.*;
import polyglot.ext.polyj.visit.*;

import polyglot.ext.jl.ast.*;

import polyglot.ast.*;
import polyglot.util.*;
import polyglot.types.*;
import polyglot.visit.*;

import java.util.*;

/**
 * A <code>PolyJBinary</code> represents a Java binary expression,
 * an immutable pair of expressions combined with an op.
 * This node behaves identically to Binary_c except in the case of
 * assignment to static parametric fields.
 */
public class PolyJBinary_c extends Binary_c implements PolyJBinary {

    /**
     * Creates a new Binary of <op> applied to <left> and <right>.
     */
    public PolyJBinary_c( Position pos, Expr left, Operator op, Expr right) {
        super( pos, left, op, right);
    }
    
    public void translate(CodeWriter w, Translator tr) {
        Context c = tr.context();
        
        PolyJTypeSystem ts = (PolyJTypeSystem)c.typeSystem();
        TranslationTable tt  = ts.translationTable();
        if (((op == EQ) || (op == NE))
            && ((left.type() instanceof ParamType)
                || (right.type() instanceof ParamType)
                || ts.equals(left.type(), ts.Object()))){
            // MRC: why is the third disjunct above necessary?
            // It doesn't seem necessary to check that case.  I'm
            // leaving it in because of the below comment, which 
            // dates back over 3 years, to previous authors.  Certainly,
            // leaving it in is just a performance hit:  at run-time,
            // Primitive.ee will do the right thing, regardless.
            // ----------------
            // When we compare two parametric objects for equality,
            // (or two objects that may be parametric),
            // we need to determine (at runtime) whether  the objects being
            // compared are primitive wrappers, in which case we should compare
            // values, not pointers.  Primitive.ee() handles this for us.
            w.write(" polyglot.ext.polyj.wrappers.Primitive.ee(");
            printSubExpr(left, w, tr);
            w.write(",");
            w.allowBreak(2);
            printSubExpr(right, w, tr);
            w.write(" ) ");
            return;
        }
    
        //This expression is not an assignment, or is not a static field,
        // or is not instantiated to a primitive type
        // ==> do as in regular Java
        super.translate(w, tr);
    }

}
