package polyglot.ext.polyj.ast;

import polyglot.ast.*;
import polyglot.types.*;
import polyglot.util.*;

import polyglot.ext.jl.ast.*;

import polyglot.ext.polyj.ast.*;
import polyglot.ext.polyj.extension.*;
import polyglot.ext.polyj.types.*;
import polyglot.ext.polyj.util.*;

import java.util.*;

/**
 * Constructs JL AST nodes with PolyJ extensions, as well as PolyJ-specific AST nodes.
 */
public class PolyJNodeFactory_c extends NodeFactory_c implements PolyJNodeFactory {
    
    public PolyJNodeFactory_c (ExtFactory ext)
    { 
	    super(ext);
    }
    
    public PolyJNodeFactory_c ()
    {
	    super(new PolyJExtFactory_c());
    }
    
    public Disamb disamb() {
        return new PolyJDisamb_c();
    }

    public JL defaultDel() {
        return new PolyJExt_c();
    }

    public ArrayAccess ArrayAccess(Position pos, Expr base, Expr index) {
       ArrayAccess n = new PolyJArrayAccess_c(pos,base,index);
       n = (ArrayAccess)n.ext(((PolyJExtFactory_c)extFactory()).extArrayAccess());        
       return n;
    }

    public LocalAssign LocalAssign(Position pos, Local left, Assign.Operator op, Expr right) {
    	PolyJLocalAssign_c n = new PolyJLocalAssign_c(pos, left, op, right);
	n = (PolyJLocalAssign_c) n.ext(((PolyJExtFactory_c)extFactory()).extLocalAssign());
	return n;
    }
    
    public FieldAssign FieldAssign(Position pos, Field left, Assign.Operator op, Expr right) {
	 PolyJFieldAssign_c n = new PolyJFieldAssign_c(pos, left, op, right);
	 n = (PolyJFieldAssign_c) n.ext(((PolyJExtFactory_c)extFactory()).extFieldAssign());
	 return n;
    }
    
    public ArrayAccessAssign ArrayAccessAssign(Position pos, ArrayAccess left, Assign.Operator op, Expr right) {
         PolyJArrayAccessAssign_c n = new PolyJArrayAccessAssign_c(pos, left, op, right);
	 n = (PolyJArrayAccessAssign_c) n.ext(((PolyJExtFactory_c)extFactory()).extArrayAccessAssign());
	 return n;
    }
		    
    public Binary Binary(Position pos, Expr left, Binary.Operator op, Expr right) {
       Binary n = new PolyJBinary_c(pos, left, op, right);
       n = (Binary)n.ext(((PolyJExtFactory_c)extFactory()).extBinary());       
       return n;       
    }

   public Call Call(Position pos, Receiver target, String name, List args) {
       Call n = new PolyJCall_c(pos, target, name, args);
       n = (Call)n.ext(((PolyJExtFactory_c)extFactory()).extCall());
      return n;	
    }

    public Cast Cast(Position pos, TypeNode type, Expr expr) {
       Cast n = new PolyJCast_c(pos, type, expr);
       n = (Cast)n.ext(((PolyJExtFactory_c)extFactory()).extCast());       
       return n;
    }

    public ClassBody ClassBody(Position pos, List members) {
       ClassBody n = new PolyJClassBody_c(pos, members);
        n = (ClassBody)n.ext(((PolyJExtFactory_c)extFactory()).extClassBody());        
         return n;	
    }
    
    public ClassDecl ClassDecl(Position pos, Flags flags, String name, TypeNode superClass, List interfaces, ClassBody body) {
      ClassDecl n = new PolyJClassDecl_c(pos, flags, name, Collections.EMPTY_LIST, Collections.EMPTY_LIST, superClass, interfaces, body);
      n = (ClassDecl) n.ext(((PolyJExtFactory_c)extFactory()).extClassDecl());      
      return n;
    }

    public ConstructorCall ConstructorCall(Position pos, 
        ConstructorCall.Kind kind, Expr outer, List args) 
    {
        PolyJConstructorCall_c n = new PolyJConstructorCall_c(pos, kind, outer, args);
	return (ConstructorCall)n.ext(((PolyJExtFactory_c)extFactory()).extConstructorCall());
    }
    
    public ConstructorDecl ConstructorDecl(Position pos, Flags flags, String name, List formals, List throwTypes, Block body) {
	PolyJConstructorDecl_c n = new PolyJConstructorDecl_c(pos, flags, name, formals, throwTypes, Collections.EMPTY_LIST, body);
	return (ConstructorDecl)n.ext(((PolyJExtFactory_c)extFactory()).extConstructorDecl());
        
    }

    public Field Field(Position pos, Receiver target, String name) {
	Field n = new PolyJField_c(pos, target, name);
        n = (Field)n.ext(((PolyJExtFactory_c)extFactory()).extFieldDecl());
        
        return n;
    }
    
    public FieldDecl FieldDecl(Position pos, Flags flags, TypeNode type, 
        String name, Expr init) 
    {
	FieldDecl n = new PolyJFieldDecl_c(pos, flags, type, name, init);
        n = (FieldDecl)n.ext(((PolyJExtFactory_c)extFactory()).extFieldDecl());
        return n;    
    }

    public Initializer Initializer(Position pos, Flags flags, Block body) {
        Initializer n = new PolyJInitializer_c(pos, flags, body);
        n = (Initializer)n.ext(((PolyJExtFactory_c)extFactory()).extInitializer());
           
        return n;      
    }
    
    public Instanceof Instanceof(Position pos, Expr expr, TypeNode type) {
        Instanceof n = new PolyJInstanceof_c(pos, expr, type);
	n = (Instanceof)n.ext(((PolyJExtFactory_c)extFactory()).extInstanceof());
        
        return n;    
    }

    public LocalDecl LocalDecl(Position pos, Flags flags, TypeNode type, String name, Expr init) {
        LocalDecl n = new PolyJLocalDecl_c(pos, flags, type, name, init);
        n = (LocalDecl)n.ext(((PolyJExtFactory_c)extFactory()).extLocalDecl());
        
        return n;    	
    }
    
    public MethodDecl MethodDecl(Position pos, Flags flags, TypeNode returnType, String name, List formals, List throwTypes, Block body) {
        PolyJMethodDecl_c n = new  PolyJMethodDecl_c(pos, flags, returnType, name, formals, throwTypes, Collections.EMPTY_LIST, body);
        return (MethodDecl)n.ext(((PolyJExtFactory_c)extFactory()).extMethodDecl());
        	    
    }

    public New New(Position pos, Expr outer, TypeNode objectType, List args, ClassBody body) {
        New n = new PolyJNew_c(pos, outer, objectType, args, body);
        n = (New)n.ext(((PolyJExtFactory_c)extFactory()).extNew());
        
        return n;	
    }

    public NewArray NewArray(Position pos, TypeNode base, List dims, int addDims, ArrayInit init) {
        NewArray n = new PolyJNewArray_c(pos, base, dims, addDims, init);
        n = (NewArray)n.ext(((PolyJExtFactory_c)extFactory()).extNewArray());
        
        return n;
    }

    public CanonicalTypeNode CanonicalTypeNode(Position pos, Type type) {
        if (! type.isCanonical()) {
            throw new InternalCompilerError("Cannot construct a canonical " +
                "type node for a non-canonical type.");
        }

	CanonicalTypeNode n = new PolyJCanonicalTypeNode_c(pos, type);
        n = (CanonicalTypeNode)n.ext(extFactory().extCanonicalTypeNode());
        
        return n;	
    }
  // PolyJ specific nodes

    public PolyJClassDecl PolyJClassDecl(Position pos, Flags flags, String name, List params, List constraints, TypeNode superClass, List interfaces, ClassBody body) {
        PolyJClassDecl_c n = new PolyJClassDecl_c(pos, flags, name, params, constraints, superClass, interfaces, body);
        return (PolyJClassDecl)n.ext(((PolyJExtFactory_c)extFactory()).extClassDecl());
            	
    }
    
    public PolyJMethodDecl PolyJMethodDecl(Position pos, Flags flags, TypeNode returnType, 
                           String name, List formals, List throwTypes, List constraints, 
                           Block body) {
			   
	PolyJMethodDecl n = new PolyJMethodDecl_c(pos, flags, returnType, name, formals,
                                     throwTypes, constraints, body);
        n = (PolyJMethodDecl)n.ext(((PolyJExtFactory_c)extFactory()).extMethodDecl());
        return n;    
    }
    
    public PolyJConstructorDecl PolyJConstructorDecl(Position pos, Flags flags, String name, 
                                List formals, List throwTypes, List constraints, 
                                Block body) {
			   
	PolyJConstructorDecl n = new PolyJConstructorDecl_c(pos, flags, name, formals,
                                          throwTypes, constraints, body);
        n = (PolyJConstructorDecl)n.ext(((PolyJExtFactory_c)extFactory()).extConstructorDecl());
        return n;   
    }
    
    public ClassConstraint ClassConstraint(Position pos, String name, List constraints) {
        ClassConstraint n = new ClassConstraint_c(pos, name, constraints);
        n = (ClassConstraint)n.ext(((PolyJExtFactory_c)extFactory()).extClassConstraint());
        return n;    
    }
    
    public MethodConstraint MethodConstraint(Position pos, Flags flags,
                                             TypeNode tn, String name,
                                             List formals, List exceptions) {
        MethodConstraint n = new MethodConstraint_c(pos, flags, tn, name, 
						formals, exceptions);
        n = (MethodConstraint)n.ext(((PolyJExtFactory_c)extFactory()).extMethodConstraint());
        return n;    
    	
    }

    public ConstructorConstraint ConstructorConstraint(Position pos, 
                                 Flags flags, String name, List formals,
                                 List exceptions) {
        ConstructorConstraint n = new ConstructorConstraint_c(pos, flags, name, formals,
                                           exceptions);
        n = (ConstructorConstraint)n.ext(((PolyJExtFactory_c)extFactory()).extConstructorConstraint());
        return n;    
   
    }

    public AmbNewArray AmbNewArray(Position pos, TypeNode baseType, Receiver param, List dims) {
        AmbNewArray n = new AmbNewArray_c(pos, baseType, param, dims, 0, null);
        n = (AmbNewArray)n.ext(((PolyJExtFactory_c)extFactory()).extAmbNewArray());
        return n;    
    
    }
    
    public AmbNewArray AmbNewArray(Position pos, TypeNode baseType, Receiver param, List dims, int addDims, ArrayInit init) {
        AmbNewArray n = new AmbNewArray_c(pos, baseType, param, dims, addDims, init);
        n = (AmbNewArray)n.ext(((PolyJExtFactory_c)extFactory()).extAmbNewArray());
        return n;  
    }
    
    public InstTypeNode InstTypeNode(Position pos, TypeNode base, List params) {
        InstTypeNode n = new InstTypeNode_c(pos, base, params);
        n = (InstTypeNode)n.ext(((PolyJExtFactory_c)extFactory()).extInstTypeNode());
        return n;
    }
    
    public ParamNode Param(Position pos, Receiver param) {
        if (param instanceof AmbReceiver) {
            return AmbNameParam(pos, (AmbReceiver) param);
        } else if (param instanceof Expr) {
            return ExprParam(pos, (Expr) param);
        } else if (param instanceof TypeNode) {
            return TypeParam(pos, (TypeNode) param);
        } else {
            throw new InternalCompilerError("Cannot construct a ParamNode from a " + param.getClass().getName(), pos);
        }
    }
    
    public AmbNameParam AmbNameParam(Position pos, AmbReceiver name) {
        AmbNameParam n = new AmbNameParam_c(pos, name);
        n = (AmbNameParam)n.ext(((PolyJExtFactory_c)extFactory()).extAmbNameParam());
        return n;
    }
    
    public ExprParam ExprParam(Position pos, Expr expr) {
        ExprParam n = new ExprParam_c(pos, expr);
        n = (ExprParam)n.ext(((PolyJExtFactory_c)extFactory()).extExprParam());
        return n;
    }
    
    public TypeParam TypeParam(Position pos, TypeNode typeNode) {
        TypeParam n = new TypeParam_c(pos, typeNode);
        n = (TypeParam)n.ext(((PolyJExtFactory_c)extFactory()).extTypeParam());
        return n;    
   }
    
    public AmbInstOrAccess AmbInstOrAccess(Position pos, Receiver prefix, Receiver param) {
        AmbInstOrAccess n = new AmbInstOrAccess_c(pos, prefix, param);
        n = (AmbInstOrAccess)n.ext(((PolyJExtFactory_c)extFactory()).extAmbInstOrAccess());
        return n;    
   }
}

