package polyglot.ext.polyj.types;

import polyglot.ext.polyj.visit.*;

import polyglot.types.*;
import polyglot.util.*;

import java.util.*;

/**
 * Each InstType keeps a list of substitutions
 * for each of the parameters to its polymorphic type.
 * additionally, it also has each of the substitutions for
 * the enclosing polymorphic classes.  Eg:
 *
 *  class A[X] { class B { class C[Y,Z] { class D { ... }}}}
 *
 *  then for the expression A[int].B.C[A[Object], W].D
 *  the enclosing class of D is a InstType with the
 *  substitutions: { X|->int, Y|->A[Object], Z|->W }
 *  (in the example I use a different letter for each parameter
 *   type, but in fact we use our usual notion of parameter equality
 *   rather than byname equality)
 *
 *  The InstType does lazy substitution.  We do no
 *  substitutions at the outset, and only perform them when someone
 *  queries us for a method, field, interface, supertype, inner class,
 *  etc.
 */
public interface InstType extends ClassType, BackMapProcedures, SubstType
{
    List actualArgs();
    PolyJClassType instType();
    boolean isNullInstantiation();
}
