package polyglot.ext.polyj.visit;

import polyglot.ast.*;
import polyglot.types.*;
import polyglot.frontend.*;
import polyglot.util.*;
import polyglot.visit.*;

import polyglot.ext.polyj.types.*;

import java.util.*;

/**
 * Checks for constraint satisfaction in all
 * type instantiations in the AST.
 */
public class InstantiationChecker extends ContextVisitor {

    protected PolyJTypeSystem ts;
    protected ErrorQueue eq;

    public InstantiationChecker(Job job, PolyJTypeSystem ts, NodeFactory nf) {
        super(job, ts, nf);
        this.ts = ts;
        this.eq = errorQueue();
    }

    public Node leaveCall(Node n) throws SemanticException {
        if (n instanceof TypeNode) {
            Type t = ((TypeNode)n).type();
            if (t instanceof InstType &&
                !((InstType)t).isNullInstantiation()) {
                PolyJClassType baseType = ((InstType)t).instType();
                List types = ((InstType)t).actualArgs();
                try {
                    ts.checkConstraints(baseType, types, context);
                } catch (ConstraintException e) {
                    List errors = e.errors();
                    for (Iterator i = errors.iterator(); i.hasNext(); ) {
                        SemanticException s = (SemanticException) i.next();
                        enqueue(s, n);
                    }
                }
            }
        }

        return n;
    }

    protected void enqueue(SemanticException e, Node n) {
        Position position = e.position();
        if (position == null) {
            position = n.position();
        }
        eq.enqueue(ErrorInfo.SEMANTIC_ERROR, e.getMessage(), position);
    }
}
